"""Formatting functions"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/api/utilities/formatting.ipynb.

# %% auto 0
__all__ = ['si_prefixes', 'significant', 'significant_digits', 'prefix_format']

# %% ../../nbs/api/utilities/formatting.ipynb
import math
from numbers import Integral

# %% ../../nbs/api/utilities/formatting.ipynb
si_prefixes = {
    -30 : 'q',
    -27 : 'r',
    -24 : 'y',
    -21 : 'z',
    -18 : 'a',
    -15 : 'f',
    -12 : 'p',
    -9  : 'n',
    -6  : 'µ',
    -3  : 'm',
    0   : '',
    3   : 'k',
    6   : 'M',
    9   : 'G',
    12  : 'T',
    15  : 'P',
    18  : 'E',
    21  : 'Z',
    24  : 'Y',
    27  : 'R',
    30  : 'Q',
}

# %% ../../nbs/api/utilities/formatting.ipynb
def significant(x, n=1):
    if x == 0: return x
    return round(x, -int(math.floor(math.log10(abs(x)))) + (n - 1))

# %% ../../nbs/api/utilities/formatting.ipynb
def significant_digits(x, n=1):
    if x == 0: return x
    x = significant(x, n)
    return int(round(x * 10**(-int(math.floor(math.log10(abs(x)))) + (n - 1))))

# %% ../../nbs/api/utilities/formatting.ipynb
def prefix_format(x, precision=3):
    if x == 0: return f'{x}'
    e = (math.floor(math.log(abs(x), 10) + 2) // 3) * 3
    m = x/10**e
    if isinstance(precision, Integral):
        p = max(precision - math.ceil(math.log(abs(m), 10)), 0)
        return f'{m:.{p}f}{si_prefixes[e]}'
    else:
        precision = significant(precision/10**e)
        p = -math.floor(math.log(precision, 10))
        if p < 0: return f'({m:.1f}±{precision:.0f}){si_prefixes[e]}'
        return f'{m:.{p}f}({significant_digits(precision)}){si_prefixes[e]}'



"""Create and manipulate a list of bins of the atomic velocity distribution for use in the LGS model"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/api/velocitygroups.ipynb.

# %% auto 0
__all__ = ['VelocityGroups', 'velocity_density_vector']

# %% ../nbs/api/velocitygroups.ipynb
from .imports import *
from .utilities.nbdev import DictTbl, AttributeTbl
from .utilities.testing import test_array
from .utilities.sparse import sparse_kronecker_matrix, sparse_toeplitz, sparse_identity, sparse_diag, sparse
from .utilities.numpy import sym_range
from .pymor.parameters import *
from .pymor.vectorarrays import *
from .pymor.operators import *
from .pymor.grids import *
from pymor.vectorarrays.interface import VectorArray
from pymor.operators.interface import Operator
from pymor.basic import IdentityOperator

# %% ../nbs/api/velocitygroups.ipynb
class VelocityGroups(dict):
    """`VelocityGroups` contains informations about the centers, edges, and relative densities of a list of velocity group bins."""
    def __init__(self, 
                 bins:Iterable|int=6, # Iterable of velocity group edges or int specifying number of evenly spaced velocity groups
                 range:Number|Iterable=3 # Default range (`min_vel`, `max_vel`) or half-width `max_vel` for velocity groups
                ):
        super().__init__()
        if isinstance(bins, VelocityGroups):
            self.edges = bins.edges
            for k, v in bins.items():
                self[k] = v
            return
        if isinstance(range, Number): range = (-range, range)
        self.edges = np.linspace(*range, bins + 1) if isinstance(bins, int) else bins
        self['VGCenter'] = np.mean(np.array(list(zip(self.edges[:-1], self.edges[1:]))), axis=1)
        # self['VGBins'] = np.array(list(zip(bins[:-1], bins[1:])))
        # self['VGCenter'] = np.mean(self['VGBins'], axis=1)
        self['VGDensity'] = np.diff(erf(self.edges))/2
        self['VGWidth'] = np.diff(self.edges)
        # self['VGEdges'] = bins
        self['VGInverseWidth'] = 1/self['VGWidth']
        self['VGNumber'] = len(self['VGCenter'])
        self['velocity_groups'] = np.ones(self['VGDensity'].shape)

# %% ../nbs/api/velocitygroups.ipynb
@patch
def subdivide(
    self:VelocityGroups, 
    indices:int|ndarray|slice=slice(None), # Indices, boolean array, or slice object specifying velocity groups to subdivide
    n:int=2 # Number of groups to subdivide into
):
    """Divide the velocity groups with indices `indices` in two."""
    new_edges = np.concat([np.linspace(c - w/2, c + w/2, n + 1)[1:-1] for c,w in zip(self["VGCenter"][indices], self["VGWidth"][indices])])
    return VelocityGroups(np.union1d(self.edges, new_edges))

# %% ../nbs/api/velocitygroups.ipynb
def _velocity_space(vg, ext=''):
    return XarrayVectorSpace({'Atomic velocity' + ext: vg['VGCenter']})

# %% ../nbs/api/velocitygroups.ipynb
def _vg_identity(vg):
    return XarrayMatrixOperator(
        DataArray(sparse_identity(vg['VGNumber']), coords=[('Atomic velocity' + Lbl.RNG, vg['VGCenter']), ('Atomic velocity', vg['VGCenter'])])
    )

# %% ../nbs/api/velocitygroups.ipynb
@patch
def identity(self:VelocityGroups)->Operator:
    """The velocity-space identity operator."""
    return IdentityOperator(_velocity_space(self))
    # return XarrayMatrixOperator(
    #         sparse_identity(self['VGNumber']), 
    #         range=_velocity_space(self), 
    #         source=_velocity_space(self)
    #     )

# %% ../nbs/api/velocitygroups.ipynb
def _vg_diagonal(vg, diags):
    return XarrayMatrixOperator(
        DataArray(sparse_diag(diags), coords=[('Atomic velocity' + Lbl.RNG, vg['VGCenter']), ('Atomic velocity' + Lbl.SRC, vg['VGCenter'])])
    )

# %% ../nbs/api/velocitygroups.ipynb
@patch
def velocity_diagonal(self:VelocityGroups)->Operator:
    """A velocity-space operator with the velocity on the diagonal."""
    return _vg_diagonal(self, self['VGCenter'])

# %% ../nbs/api/velocitygroups.ipynb
def velocity_density_vector(vg):
    return _velocity_space(vg).from_numpy(vg['VGDensity'])

# %% ../nbs/api/velocitygroups.ipynb
@patch
def n_times_1(self:VelocityGroups)->Operator:
    """Operator that sums over all velocity groups then scales by the Maxwell-Boltzmann distribution."""
    return XarrayMatrixOperator(
        sparse(self['VGDensity'][:, None] * np.ones((self['VGNumber']))),
        range=_velocity_space(self), 
        source=_velocity_space(self)
    )

# %% ../nbs/api/velocitygroups.ipynb
@patch
def drho_dv(self:VelocityGroups)->Operator:
    """Derivative with respect to velocity operator.$"""
    return XarrayMatrixOperator(
        self['VGInverseWidth'] * (sparse_kronecker_matrix(self['VGNumber'], 0) - sparse_kronecker_matrix(self['VGNumber'], -1)), 
        range=_velocity_space(self), 
        source=_velocity_space(self)
    )

# %% ../nbs/api/velocitygroups.ipynb
def _vg_da(a, vg):
    range = ('Atomic velocity' + Lbl.RNG, vg['VGCenter']) if a.shape[0] == vg['VGNumber'] else ("none", ["none"])
    source = ('Atomic velocity' + Lbl.SRC, vg['VGCenter']) if a.shape[1] == vg['VGNumber'] else ("none", ["none"])
    return DataArray(a, coords=[range, source])

# %% ../nbs/api/velocitygroups.ipynb
@patch
def normalize(self:VelocityGroups)->Operator:
    """Returns the operator that normalizes a vector by dividing each component by the width of the corresponding velocity group."""
    return ScaleOperator(DataArray(self['VGInverseWidth'], {'Atomic velocity': self['VGCenter']}))

# %% ../nbs/api/velocitygroups.ipynb
@patch
def sum(self:VelocityGroups)->Operator:
    """Returns the operator that sums a vector over velocity groups."""
    return SumOperator({'Atomic velocity': self['VGCenter']})



"""Extended functionality for [pyMOR](https://pymor.org/) models"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/api/pymor/models.ipynb.

# %% auto 0
__all__ = ['StationaryModel', 'InstationaryModel', 'fourier_space', 'to_sympy', 'fourier_series', 'fourier_series_coefficients',
           'fourier_expansion_operator', 'FourierExpandRules', 'fourier_expand', 'fourier_diagonal_operator', 'd_term',
           'floquet_expand', 'time_space', 'floquet_reconstruction_array', 'floquet_reconstruction_operator',
           'StationaryFloquetModel']

# %% ../../nbs/api/pymor/models.ipynb
from functools import partial
from numbers import Number

from fastcore.basics import patch, first
import pymor
from pymor.basic import Mu, LincombOperator, IdentityOperator
from pymor.vectorarrays.interface import VectorArray
from ..utilities.sparse import sparse_diag, sparse_identity

# %% ../../nbs/api/pymor/models.ipynb
class StationaryModel(pymor.basic.StationaryModel):
    """Extend pyMOR's `StationaryModel` to include a `data` attribute for storing, e.g., the model's grid."""
    def __init__(
        self,
        operator,
        rhs,
        output_functional=None,
        products=None,
        error_estimator=None,
        visualizer=None,
        name=None,
        data=None
    ):
        self.data = data
        if products:
            for k, v in products.items():
                setattr(self, k, partial(self._apply_product, v=v))
                # setattr(self, k, lambda u, mu=None, v=v: v.apply(u, Mu(mu) if mu is not None else None) if isinstance(u, VectorArray) else v.apply(self.solve(u), mu=Mu(u)))
        super().__init__(
            operator,
            rhs,
            output_functional=output_functional,
            products=products,
            error_estimator=error_estimator,
            visualizer=visualizer,
            name=name,
        )

    def _apply_product(self, u, mu=None, v=None):
        if mu is not None: mu = Mu(mu)
        if isinstance(u, VectorArray):
            return v.apply(u, mu)
        return v.apply(self.solve(u), mu=mu)

# %% ../../nbs/api/pymor/models.ipynb
@patch
def solve(self:StationaryModel, mu=None, input=None, return_error_estimate=False, **kwargs):
    """Extend `StationaryModel.solve` to solve over a range of parameters."""
    if not isinstance(mu, (list, tuple)):
        data = self.compute(
            solution=True,
            solution_error_estimate=return_error_estimate,
            mu=mu,
            input=input,
            **kwargs
        )
        if return_error_estimate:
            return data['solution'], data['solution_error_estimate']
        else:
            return data['solution']
    else:
        fixed_mu, var_mu = mu
        n = len(first(var_mu.values()))
        if not all(len(v) == n for v in var_mu.values()): raise ValueError(f'All varying parameter lists {var_mu} must be the same length.')
        sol = [self.solve(fixed_mu | {k: v[i] for k, v in var_mu.items()}, input=input, return_error_estimate=False, **kwargs) for i in range(n)]
        return self.operator.source.concatenate(sol, var_mu)

# %% ../../nbs/api/pymor/models.ipynb
class InstationaryModel(pymor.basic.InstationaryModel):
    """Extend pyMOR's `InstationaryModel` to include a `data` attribute for storing, e.g., the model's grid."""
    def __init__(
        self,
        T,
        initial_data,
        operator,
        rhs,
        mass=None,
        time_stepper=None,
        num_values=None,
        output_functional=None,
        products=None,
        error_estimator=None,
        visualizer=None,
        name=None,
        data=None
    ):
        self.data = data
        if products:
            for k, v in products.items():
                setattr(self, k, lambda u, mu=None, v=v: v.apply(u, mu))
        super().__init__(
        T,
        initial_data,
        operator,
        rhs,
        mass=mass,
        time_stepper=time_stepper,
        num_values=num_values,
        output_functional=output_functional,
        products=products,
        error_estimator=error_estimator,
        visualizer=visualizer,
        name=name,
    )

# %% ../../nbs/api/pymor/models.ipynb
import math

from ..utilities.numpy import sym_range
from ..utilities.sparse import sparse_toeplitz, dense_toeplitz
from .parameters import simplify_functionals
from .vectorarrays import XarrayVectorSpace
from .operators import XarrayMatrixOperator, XarrayFunctionalOperator
from pymor.basic import ExpressionParameterFunctional, GenericParameterFunctional
from pymor.parameters.functionals import ProductParameterFunctional
from pymor.algorithms.simplify import expand
from xarray import Coordinates, DataArray
import numpy as np
from numpy import array
import sympy as sy
from sympy import Expr
from sympy.parsing.sympy_parser import parse_expr

# %% ../../nbs/api/pymor/models.ipynb
def fourier_space(n): 
    return XarrayVectorSpace({'Fourier coefficient': sym_range(n)})

# %% ../../nbs/api/pymor/models.ipynb
def to_sympy(functional):
    if isinstance(functional, Number):
        return functional
    if isinstance(functional, ProductParameterFunctional):
        return math.prod(to_sympy(factor) for factor in functional.factors)
    if isinstance(functional, ExpressionParameterFunctional): 
        out = functional.expression
    else:
        out = functional
    return parse_expr(out)

# %% ../../nbs/api/pymor/models.ipynb
def fourier_series(expr, n, t:str='t', nu:str='nu'):
    t = sy.Symbol(t)
    return sy.fourier_series(
        expr.subs(nu, 1), 
        (t, -sy.pi, sy.pi)
    ).truncate(
        n=n+1
    ).rewrite(
        sy.exp
    ).expand()

# %% ../../nbs/api/pymor/models.ipynb
def fourier_series_coefficients(series:Expr, n:int, t:str='t'):
    """Returns the positive and negative Fourier series coefficients up to and including order `n`.
    """
    t = sy.Symbol(t)
    return array([
        series.coeff(sy.exp(sy.I*t) if k != 0 else t, n=k) 
        for k in range(-n, n+1)
    ])

# %% ../../nbs/api/pymor/models.ipynb
def fourier_expansion_operator(coefficient, n_operator, n_vector, mu:dict=None, t:str='t', nu:str='nu') -> np.ndarray:
    """Coefficients of complex Fourier series of expr in variable `t` at frequency `nu`.

    Assumes that either the coefficients are numerical, or there is only the zero-order coefficient. Could be useful to generalize.
    """
    space = fourier_space(n_vector)
    if isinstance(coefficient, Number):
        return IdentityOperator(space) * coefficient        
    expr = to_sympy(coefficient)
    if mu is not None: expr = expr.subs(mu)
    if not expr.has(sy.Symbol(t)): 
        return IdentityOperator(space) * coefficient
    series = fourier_series(expr, n_operator, t=t, nu=nu)
    coeffs = fourier_series_coefficients(series, n_operator, t=t).astype(complex)
    return XarrayMatrixOperator(
        sparse_toeplitz(np.pad(coeffs, n_vector + 1)),
        source=space,
        range=space
    )

# %% ../../nbs/api/pymor/models.ipynb
from pymor.algorithms.rules import match_class, RuleTable
from pymor.models.interface import Model
from pymor.operators.interface import Operator

# %% ../../nbs/api/pymor/models.ipynb
class FourierExpandRules(RuleTable):
    """|RuleTable| to Fourier expand an operator."""

    def __init__(self, n_operator:int, n_vector:int, mu:dict|None=None, t:str='t', nu:str='nu'):
        super().__init__(use_caching=True)
        self.__auto_init(locals())

    @match_class(LincombOperator)
    def action_LincombOperator(self, op):
        if not set(self.mu).intersection(op.parameters): return op * IdentityOperator(fourier_space(self.n_vector))
        # op = self.replace_children(op)
        return sum(
            fourier_expansion_operator(c, self.n_operator, self.n_vector, mu=self.mu, t=self.t, nu=self.nu) * o
            for c, o in zip(op.coefficients, op.operators)
        )

    @match_class(Model, Operator)
    def action_recurse(self, op):
        if not set(self.mu).intersection(op.parameters): return op * IdentityOperator(fourier_space(self.n_vector))
        return self.replace_children(op)

# %% ../../nbs/api/pymor/models.ipynb
def fourier_expand(obj, n_operator:int, n_vector:int, mu:dict|None=None, t:str='t', nu:str='nu'):
    return FourierExpandRules(n_operator, n_vector, mu=mu).apply(obj)

# %% ../../nbs/api/pymor/models.ipynb
def fourier_diagonal_operator(vec):
    space = fourier_space((len(vec)-1)//2)
    return XarrayMatrixOperator(sparse_diag(vec), source=space, range=space)

# %% ../../nbs/api/pymor/models.ipynb
def d_term(n_vector:int, nu:str='nu'):
    return ExpressionParameterFunctional(f'-(1j)*{nu}', {nu: 1}) * fourier_diagonal_operator(sym_range(n_vector))

# %% ../../nbs/api/pymor/models.ipynb
def floquet_expand(op, n_operator, n_vector, mu:dict=None, t:str='t', nu:str='nu'):
    return (
        d_term(n_vector, nu=nu) * IdentityOperator(op.source)
        + fourier_expand(op, n_operator, n_vector, mu=mu, t=t, nu=nu)
    ).with_(solver=op.solver)

# %% ../../nbs/api/pymor/models.ipynb
def _kron_vector(space): return space.from_numpy((sym_range((space.dim - 1)//2) == 0).astype(int))

# %% ../../nbs/api/pymor/models.ipynb
def time_space(time): return XarrayVectorSpace({'Time': time})

# %% ../../nbs/api/pymor/models.ipynb
def floquet_reconstruction_array(n_vector, time, mu):
    time_coord = time_space(time).coords['Time']
    fourier_coord = fourier_space(n_vector).coords['Fourier coefficient']
    nu = ExpressionParameterFunctional('nu', {'nu': 1}).evaluate(mu)
    return np.exp(-1j * nu * time_coord * fourier_coord)

# %% ../../nbs/api/pymor/models.ipynb
def floquet_reconstruction_operator(n_vector, time):
    return XarrayFunctionalOperator(
        GenericParameterFunctional(lambda mu: floquet_reconstruction_array(n_vector, time, mu), {'nu': 1}),
        range=time_space(time),
        source=fourier_space(n_vector),
    )

# %% ../../nbs/api/pymor/models.ipynb
class StationaryFloquetModel(StationaryModel):
    """A model that solves for the Fourier coefficients of the periodic state of a modulated system."""

    def __init__(
        self,
        operator, # System evolution operator
        rhs, # System right-hand side vector
        mu, # Parameter specification for parameters modulated at a frequency `nu` 
        n_vector, # Number of terms in the system Fourier series
        time, # Time values for displaying result
        nu='nu', # Name of the frequency variable
        t='t', # Name of the time variable
        n_operator=None, # Number of terms in the operator Fourier series. `None` defaults to `n_vector` - 1
        products=None,
        name=None,
        data=None,
        solver=None,
        preconditioner=None,
    ):
        if n_operator is None: n_operator = n_vector - 1
        operator = floquet_expand(expand(operator), n_operator, n_vector, mu=mu, t=t, nu=nu)
        if solver is not None:
            if preconditioner is not None:
                preconditioner = floquet_expand(expand(preconditioner), n_operator, n_vector, mu=mu, t=t, nu=nu)
                solver = solver.with_(preconditioner=preconditioner)
            operator = operator.with_(solver=solver)
        rhs = _kron_vector(fourier_space(n_vector)) * rhs
        super().__init__(
            operator=operator,
            rhs=rhs,
            products=(products or {}) | self._floquet_products(n_vector, time),
            data=data
        )
        self.__auto_init(locals())

    @staticmethod
    def _floquet_products(n_vector, time): 
        return {
            'reconstruct': floquet_reconstruction_operator(n_vector, time),
        }



"""Operators on density-matrix elements"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/api/dmelements.ipynb.

# %% auto 0
__all__ = ['parse_dm_element', 'population_element', 'level_label', 'levels', 'population', 'total_population', 'level_mask_da',
           'level_population_mask_da', 'level_population', 'sum_over_dm']

# %% ../nbs/api/dmelements.ipynb
import importlib

from .imports import *
from .utilities.nbdev import DictTbl, AttributeTbl
from .utilities.testing import test_array
from .utilities.sparse import sparse_kronecker_matrix, sparse_toeplitz, sparse_identity, sparse_diag, sparse
from .utilities.numpy import sym_range
from .pymor.parameters import *
from .pymor.vectorarrays import *
from .pymor.operators import *
from .pymor.timestepping import *
from .pymor.grids import *
from .pymor.models import *
from pymor.vectorarrays.interface import VectorArray
from pymor.operators.interface import Operator

# %% ../nbs/api/dmelements.ipynb
_dm_element_hfz = re.compile(r'ρ<sub>([^,]+), \(([^,]+), ([^,]+), ([^)]+)\), \(([^,]+), ([^,]+), ([^)]+)\)</sub>')
_dm_element_toy = re.compile(r'ρ<sub>([^,]+), ([^,]+), ([^,]+)</sub>')

# %% ../nbs/api/dmelements.ipynb
def parse_dm_element(s):
    if match := _dm_element_hfz.findall(s):
        return match[0][0], match[0][1:4], match[0][4:7] 
    if match := _dm_element_toy.findall(s): 
        return match[0][0], (match[0][1],), (match[0][2],)

# %% ../nbs/api/dmelements.ipynb
@dispatch
def population_element(s):
    parsed_dm = parse_dm_element(s)
    return parsed_dm[1] == parsed_dm[2]

@dispatch
def population_element(expr:DataArray):
    return xr.apply_ufunc(population_element, expr, vectorize=True)

# %% ../nbs/api/dmelements.ipynb
@dispatch
def level_label(expr:str):
    parsed = parse_dm_element(expr)
    if parsed[1][0] != parsed[2][0]: return ''
    return parsed[1][0]

@dispatch
def level_label(expr:DataArray):
    return xr.apply_ufunc(level_label, expr, vectorize=True)

# %% ../nbs/api/dmelements.ipynb
def levels(dm_elements):
    return xr.Variable(
        'Level', 
        list(set().union(filter(None, xr.apply_ufunc(level_label, dm_elements, vectorize=True).data)))
    )

# %% ../nbs/api/dmelements.ipynb
def population(dm_elements):
    elements = dm_elements.data
    mask = [population_element(e) for e in elements]
    pop_labels = [level_label(e) for e in elements[mask]]
    return XarrayMatrixOperator(
        DataArray(
            np.identity(len(elements))[mask],
            coords=[
                ("Population (range)", elements[mask]),
                ("Density matrix (source)", elements), 
            ],
        )
    )

# %% ../nbs/api/dmelements.ipynb
def total_population(dm_elements):
    return XarrayMatrixOperator(
        population(
            dm_elements
        ).matrix.sum(
            "Population (range)"
        ),
        name="Total population"
    )

# %% ../nbs/api/dmelements.ipynb
def level_mask_da(dm_elements):
    return xr.concat([level == level_label(dm_elements) for level in levels(dm_elements)], levels(dm_elements))

# %% ../nbs/api/dmelements.ipynb
def level_population_mask_da(dm_elements):
    return level_mask_da(dm_elements) * population_element(dm_elements)

# %% ../nbs/api/dmelements.ipynb
def level_population(dm_elements):
    return XarrayMatrixOperator(
            level_population_mask_da(dm_elements),
            source="Density matrix",
            range="Level",
            name="Population"
        )

# %% ../nbs/api/dmelements.ipynb
def sum_over_dm(dm_elements):
    return SumOperator({'Density matrix' + Lbl.SRC: dm_elements})



"""Extra functionality for [xarray](https://xarray.pydata.org/)"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/api/utilities/xarray.ipynb.

# %% auto 0
__all__ = []

# %% ../../nbs/api/utilities/xarray.ipynb
from fastcore.basics import patch
from xarray import Coordinates
import numpy as np

# %% ../../nbs/api/utilities/xarray.ipynb
@patch
def __add__(self:Coordinates, other):
    """Adding two `Coordinates` objects combines their coordinates."""
    if not isinstance(other, Coordinates): raise TypeError
    result = self.copy()
    result.update(other)
    return result

# %% ../../nbs/api/utilities/xarray.ipynb
@patch
def complement(self:Coordinates, other:Coordinates):
    """Return coordinates not in other coordinates."""
    return Coordinates({k: v for k, v in self.items() if k not in other})

# %% ../../nbs/api/utilities/xarray.ipynb
@patch(as_prop=True)
def shape(self:Coordinates):
    """Return tuple of sizes of the coordinates."""
    return tuple(self.sizes.values())

# %% ../../nbs/api/utilities/xarray.ipynb
@patch(as_prop=True)
def size(self:Coordinates):
    """Return product of coordinate lengths."""
    return np.prod(self.shape)

# %% ../../nbs/api/utilities/xarray.ipynb
@patch
def intersection(self:Coordinates, other:Coordinates):
    """Return coordinates in self and other."""
    return Coordinates({k: v for k, v in self.items() if k in other and v.equals(other[k])})

# %% ../../nbs/api/utilities/xarray.ipynb
@patch
def contain(self:Coordinates, other:Coordinates):
    """Return true if all coordinates in `other` are in `self`, otherwise false."""
    return self.intersection(other).equals(other)



from .imports import *
from .utilities import *
from .pymor.timestepping import *
from .pymor.operators import *
from .pymor.vectorarrays import *
from .velocitygroups import *
from .dmelements import *
from .lgssystem import *


# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/pylgs',
                'doc_host': 'https://simonkeys.github.io',
                'git_url': 'https://github.com/simonkeys/pylgs',
                'lib_path': 'pylgs'},
  'syms': { 'pylgs.all': {},
            'pylgs.dmelements': { 'pylgs.dmelements.level_label': ('api/dmelements.html#level_label', 'pylgs/dmelements.py'),
                                  'pylgs.dmelements.level_mask_da': ('api/dmelements.html#level_mask_da', 'pylgs/dmelements.py'),
                                  'pylgs.dmelements.level_population': ('api/dmelements.html#level_population', 'pylgs/dmelements.py'),
                                  'pylgs.dmelements.level_population_mask_da': ( 'api/dmelements.html#level_population_mask_da',
                                                                                 'pylgs/dmelements.py'),
                                  'pylgs.dmelements.levels': ('api/dmelements.html#levels', 'pylgs/dmelements.py'),
                                  'pylgs.dmelements.parse_dm_element': ('api/dmelements.html#parse_dm_element', 'pylgs/dmelements.py'),
                                  'pylgs.dmelements.population': ('api/dmelements.html#population', 'pylgs/dmelements.py'),
                                  'pylgs.dmelements.population_element': ('api/dmelements.html#population_element', 'pylgs/dmelements.py'),
                                  'pylgs.dmelements.sum_over_dm': ('api/dmelements.html#sum_over_dm', 'pylgs/dmelements.py'),
                                  'pylgs.dmelements.total_population': ('api/dmelements.html#total_population', 'pylgs/dmelements.py')},
            'pylgs.imports': {},
            'pylgs.lgssystem': { 'pylgs.lgssystem.LGSSystem': ('api/lgssystem.html#lgssystem', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.__init__': ('api/lgssystem.html#lgssystem.__init__', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem._builtin_paths': ( 'api/lgssystem.html#lgssystem._builtin_paths',
                                                                               'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem._floquet_products': ( 'api/lgssystem.html#lgssystem._floquet_products',
                                                                                  'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem._get_path': ('api/lgssystem.html#lgssystem._get_path', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem._initial_dm': ( 'api/lgssystem.html#lgssystem._initial_dm',
                                                                            'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem._is_valid_dir': ( 'api/lgssystem.html#lgssystem._is_valid_dir',
                                                                              'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem._products': ('api/lgssystem.html#lgssystem._products', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem._reaction': ('api/lgssystem.html#lgssystem._reaction', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem._recoil': ('api/lgssystem.html#lgssystem._recoil', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem._vcc': ('api/lgssystem.html#lgssystem._vcc', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.adaptive_stationary_floquet_model': ( 'api/lgssystem.html#lgssystem.adaptive_stationary_floquet_model',
                                                                                                  'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.adaptive_stationary_model': ( 'api/lgssystem.html#lgssystem.adaptive_stationary_model',
                                                                                          'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.builtins': ('api/lgssystem.html#lgssystem.builtins', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.diagram': ('api/lgssystem.html#lgssystem.diagram', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.info': ('api/lgssystem.html#lgssystem.info', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.instationary_model': ( 'api/lgssystem.html#lgssystem.instationary_model',
                                                                                   'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.numpy_instationary_model': ( 'api/lgssystem.html#lgssystem.numpy_instationary_model',
                                                                                         'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.operator': ('api/lgssystem.html#lgssystem.operator', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.rhs': ('api/lgssystem.html#lgssystem.rhs', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.stationary_floquet_model': ( 'api/lgssystem.html#lgssystem.stationary_floquet_model',
                                                                                         'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.LGSSystem.stationary_model': ( 'api/lgssystem.html#lgssystem.stationary_model',
                                                                                 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.ScipyGCROTMKSpILUSolver': ( 'api/lgssystem.html#scipygcrotmkspilusolver',
                                                                              'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.ScipyGCROTMKSpILUSolver.__init__': ( 'api/lgssystem.html#scipygcrotmkspilusolver.__init__',
                                                                                       'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.ScipyGCROTMKSpILUSolver._solve': ( 'api/lgssystem.html#scipygcrotmkspilusolver._solve',
                                                                                     'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem._to_nmo': ('api/lgssystem.html#_to_nmo', 'pylgs/lgssystem.py'),
                                 'pylgs.lgssystem.optree': ('api/lgssystem.html#optree', 'pylgs/lgssystem.py')},
            'pylgs.pymor.grids': { 'pylgs.pymor.grids.OnedUnstructuredGrid': ( 'api/pymor/grids.html#onedunstructuredgrid',
                                                                               'pylgs/pymor/grids.py'),
                                   'pylgs.pymor.grids.OnedUnstructuredGrid.__init__': ( 'api/pymor/grids.html#onedunstructuredgrid.__init__',
                                                                                        'pylgs/pymor/grids.py'),
                                   'pylgs.pymor.grids.OnedUnstructuredGrid.embeddings': ( 'api/pymor/grids.html#onedunstructuredgrid.embeddings',
                                                                                          'pylgs/pymor/grids.py'),
                                   'pylgs.pymor.grids.OnedUnstructuredGrid.subentities': ( 'api/pymor/grids.html#onedunstructuredgrid.subentities',
                                                                                           'pylgs/pymor/grids.py')},
            'pylgs.pymor.models': { 'pylgs.pymor.models.FourierExpandRules': ( 'api/pymor/models.html#fourierexpandrules',
                                                                               'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.FourierExpandRules.__init__': ( 'api/pymor/models.html#fourierexpandrules.__init__',
                                                                                        'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.FourierExpandRules.action_LincombOperator': ( 'api/pymor/models.html#fourierexpandrules.action_lincomboperator',
                                                                                                      'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.FourierExpandRules.action_recurse': ( 'api/pymor/models.html#fourierexpandrules.action_recurse',
                                                                                              'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.InstationaryModel': ( 'api/pymor/models.html#instationarymodel',
                                                                              'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.InstationaryModel.__init__': ( 'api/pymor/models.html#instationarymodel.__init__',
                                                                                       'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.StationaryFloquetModel': ( 'api/pymor/models.html#stationaryfloquetmodel',
                                                                                   'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.StationaryFloquetModel.__init__': ( 'api/pymor/models.html#stationaryfloquetmodel.__init__',
                                                                                            'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.StationaryFloquetModel._floquet_products': ( 'api/pymor/models.html#stationaryfloquetmodel._floquet_products',
                                                                                                     'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.StationaryModel': ( 'api/pymor/models.html#stationarymodel',
                                                                            'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.StationaryModel.__init__': ( 'api/pymor/models.html#stationarymodel.__init__',
                                                                                     'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.StationaryModel._apply_product': ( 'api/pymor/models.html#stationarymodel._apply_product',
                                                                                           'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.StationaryModel.solve': ( 'api/pymor/models.html#stationarymodel.solve',
                                                                                  'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models._kron_vector': ('api/pymor/models.html#_kron_vector', 'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.d_term': ('api/pymor/models.html#d_term', 'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.floquet_expand': ('api/pymor/models.html#floquet_expand', 'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.floquet_reconstruction_array': ( 'api/pymor/models.html#floquet_reconstruction_array',
                                                                                         'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.floquet_reconstruction_operator': ( 'api/pymor/models.html#floquet_reconstruction_operator',
                                                                                            'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.fourier_diagonal_operator': ( 'api/pymor/models.html#fourier_diagonal_operator',
                                                                                      'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.fourier_expand': ('api/pymor/models.html#fourier_expand', 'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.fourier_expansion_operator': ( 'api/pymor/models.html#fourier_expansion_operator',
                                                                                       'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.fourier_series': ('api/pymor/models.html#fourier_series', 'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.fourier_series_coefficients': ( 'api/pymor/models.html#fourier_series_coefficients',
                                                                                        'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.fourier_space': ('api/pymor/models.html#fourier_space', 'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.time_space': ('api/pymor/models.html#time_space', 'pylgs/pymor/models.py'),
                                    'pylgs.pymor.models.to_sympy': ('api/pymor/models.html#to_sympy', 'pylgs/pymor/models.py')},
            'pylgs.pymor.operators': { 'pylgs.pymor.operators.DensifyRules': ( 'api/pymor/operators.html#densifyrules',
                                                                               'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.DensifyRules.__init__': ( 'api/pymor/operators.html#densifyrules.__init__',
                                                                                        'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.DensifyRules.action_XarrayMatrixOperator': ( 'api/pymor/operators.html#densifyrules.action_xarraymatrixoperator',
                                                                                                           'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.DensifyRules.action_recurse': ( 'api/pymor/operators.html#densifyrules.action_recurse',
                                                                                              'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ExpressionParameterFunctional.partial_evaluate': ( 'api/pymor/operators.html#expressionparameterfunctional.partial_evaluate',
                                                                                                                 'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.IdentityOperator.__str__': ( 'api/pymor/operators.html#identityoperator.__str__',
                                                                                           'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.LincombOperator.__init__': ( 'api/pymor/operators.html#lincomboperator.__init__',
                                                                                           'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.LincombOperator.from_file': ( 'api/pymor/operators.html#lincomboperator.from_file',
                                                                                            'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.LincombOperator.partial_assemble': ( 'api/pymor/operators.html#lincomboperator.partial_assemble',
                                                                                                   'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.LincombOperator.partial_evaluate_coefficients': ( 'api/pymor/operators.html#lincomboperator.partial_evaluate_coefficients',
                                                                                                                'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.LincombOperator.terms': ( 'api/pymor/operators.html#lincomboperator.terms',
                                                                                        'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.Operator.__mul__': ( 'api/pymor/operators.html#operator.__mul__',
                                                                                   'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.Operator._repr_html_': ( 'api/pymor/operators.html#operator._repr_html_',
                                                                                       'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ProductOperator': ( 'api/pymor/operators.html#productoperator',
                                                                                  'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ProductOperator.H': ( 'api/pymor/operators.html#productoperator.h',
                                                                                    'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ProductOperator.__init__': ( 'api/pymor/operators.html#productoperator.__init__',
                                                                                           'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ProductOperator.__str__': ( 'api/pymor/operators.html#productoperator.__str__',
                                                                                          'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ProductOperator.apply': ( 'api/pymor/operators.html#productoperator.apply',
                                                                                        'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ProductOperator.apply_adjoint': ( 'api/pymor/operators.html#productoperator.apply_adjoint',
                                                                                                'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ProductOperator.as_range_array': ( 'api/pymor/operators.html#productoperator.as_range_array',
                                                                                                 'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ProductOperator.as_source_array': ( 'api/pymor/operators.html#productoperator.as_source_array',
                                                                                                  'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ProductOperator.assemble': ( 'api/pymor/operators.html#productoperator.assemble',
                                                                                           'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ScaleOperator': ( 'api/pymor/operators.html#scaleoperator',
                                                                                'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ScaleOperator.__init__': ( 'api/pymor/operators.html#scaleoperator.__init__',
                                                                                         'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ScaleOperator.__mul__': ( 'api/pymor/operators.html#scaleoperator.__mul__',
                                                                                        'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ScaleOperator.__rmul__': ( 'api/pymor/operators.html#scaleoperator.__rmul__',
                                                                                         'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ScaleOperator.__str__': ( 'api/pymor/operators.html#scaleoperator.__str__',
                                                                                        'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ScaleOperator.apply': ( 'api/pymor/operators.html#scaleoperator.apply',
                                                                                      'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ScaleOperator.assemble': ( 'api/pymor/operators.html#scaleoperator.assemble',
                                                                                         'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.ScaleOperator.display_name': ( 'api/pymor/operators.html#scaleoperator.display_name',
                                                                                             'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.SumOperator': ( 'api/pymor/operators.html#sumoperator',
                                                                              'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.SumOperator.__init__': ( 'api/pymor/operators.html#sumoperator.__init__',
                                                                                       'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.SumOperator.__mul__': ( 'api/pymor/operators.html#sumoperator.__mul__',
                                                                                      'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.SumOperator.__rmul__': ( 'api/pymor/operators.html#sumoperator.__rmul__',
                                                                                       'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.SumOperator.__str__': ( 'api/pymor/operators.html#sumoperator.__str__',
                                                                                      'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.SumOperator.apply': ( 'api/pymor/operators.html#sumoperator.apply',
                                                                                    'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.SumOperator.assemble': ( 'api/pymor/operators.html#sumoperator.assemble',
                                                                                       'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayFunctionalOperator': ( 'api/pymor/operators.html#xarrayfunctionaloperator',
                                                                                           'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayFunctionalOperator.__init__': ( 'api/pymor/operators.html#xarrayfunctionaloperator.__init__',
                                                                                                    'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayFunctionalOperator.apply': ( 'api/pymor/operators.html#xarrayfunctionaloperator.apply',
                                                                                                 'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayFunctionalOperator.assemble': ( 'api/pymor/operators.html#xarrayfunctionaloperator.assemble',
                                                                                                    'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayFunctionalOperator.size': ( 'api/pymor/operators.html#xarrayfunctionaloperator.size',
                                                                                                'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixBasedOperator': ( 'api/pymor/operators.html#xarraymatrixbasedoperator',
                                                                                            'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixBasedOperator.H': ( 'api/pymor/operators.html#xarraymatrixbasedoperator.h',
                                                                                              'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixBasedOperator.__str__': ( 'api/pymor/operators.html#xarraymatrixbasedoperator.__str__',
                                                                                                    'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixBasedOperator.apply': ( 'api/pymor/operators.html#xarraymatrixbasedoperator.apply',
                                                                                                  'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixBasedOperator.apply_adjoint': ( 'api/pymor/operators.html#xarraymatrixbasedoperator.apply_adjoint',
                                                                                                          'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixBasedOperator.as_range_array': ( 'api/pymor/operators.html#xarraymatrixbasedoperator.as_range_array',
                                                                                                           'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixBasedOperator.as_source_array': ( 'api/pymor/operators.html#xarraymatrixbasedoperator.as_source_array',
                                                                                                            'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixBasedOperator.assemble': ( 'api/pymor/operators.html#xarraymatrixbasedoperator.assemble',
                                                                                                     'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixBasedOperator.display_name': ( 'api/pymor/operators.html#xarraymatrixbasedoperator.display_name',
                                                                                                         'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator': ( 'api/pymor/operators.html#xarraymatrixoperator',
                                                                                       'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator.H': ( 'api/pymor/operators.html#xarraymatrixoperator.h',
                                                                                         'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator.__init__': ( 'api/pymor/operators.html#xarraymatrixoperator.__init__',
                                                                                                'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator._assemble_lincomb': ( 'api/pymor/operators.html#xarraymatrixoperator._assemble_lincomb',
                                                                                                         'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator.apply': ( 'api/pymor/operators.html#xarraymatrixoperator.apply',
                                                                                             'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator.apply_adjoint': ( 'api/pymor/operators.html#xarraymatrixoperator.apply_adjoint',
                                                                                                     'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator.assemble': ( 'api/pymor/operators.html#xarraymatrixoperator.assemble',
                                                                                                'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator.size': ( 'api/pymor/operators.html#xarraymatrixoperator.size',
                                                                                            'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator.sparse': ( 'api/pymor/operators.html#xarraymatrixoperator.sparse',
                                                                                              'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayMatrixOperator.to_numpy': ( 'api/pymor/operators.html#xarraymatrixoperator.to_numpy',
                                                                                                'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayVectorSpace.__truediv__': ( 'api/pymor/operators.html#xarrayvectorspace.__truediv__',
                                                                                                'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.XarrayVectorSpace.range_under': ( 'api/pymor/operators.html#xarrayvectorspace.range_under',
                                                                                                'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._get_space_dims': ( 'api/pymor/operators.html#_get_space_dims',
                                                                                  'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._is_labeled_da': ( 'api/pymor/operators.html#_is_labeled_da',
                                                                                 'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._label_dims': ( 'api/pymor/operators.html#_label_dims',
                                                                              'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._label_dims_with': ( 'api/pymor/operators.html#_label_dims_with',
                                                                                   'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._mapping_str': ( 'api/pymor/operators.html#_mapping_str',
                                                                               'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._ndims': ('api/pymor/operators.html#_ndims', 'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._numpy_zero_operator': ( 'api/pymor/operators.html#_numpy_zero_operator',
                                                                                       'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._operator_group_tag': ( 'api/pymor/operators.html#_operator_group_tag',
                                                                                      'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._operator_html_node': ( 'api/pymor/operators.html#_operator_html_node',
                                                                                      'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._read_sparse_dataarray': ( 'api/pymor/operators.html#_read_sparse_dataarray',
                                                                                         'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators._swap_dim_labels': ( 'api/pymor/operators.html#_swap_dim_labels',
                                                                                   'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.contract_ProductOperator': ( 'api/pymor/operators.html#contract_productoperator',
                                                                                           'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.contract_matvec': ( 'api/pymor/operators.html#contract_matvec',
                                                                                  'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.coordinates_to_list': ( 'api/pymor/operators.html#coordinates_to_list',
                                                                                      'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.densify': ('api/pymor/operators.html#densify', 'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.dims_matvec': ( 'api/pymor/operators.html#dims_matvec',
                                                                              'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.expand_IdentityOperator': ( 'api/pymor/operators.html#expand_identityoperator',
                                                                                          'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.expand_ProductOperator': ( 'api/pymor/operators.html#expand_productoperator',
                                                                                         'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.operator_html_tree': ( 'api/pymor/operators.html#operator_html_tree',
                                                                                     'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.replace_item': ( 'api/pymor/operators.html#replace_item',
                                                                               'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.to_matrix_ProductOperator': ( 'api/pymor/operators.html#to_matrix_productoperator',
                                                                                            'pylgs/pymor/operators.py'),
                                       'pylgs.pymor.operators.to_matrix_XarrayMatrixOperator': ( 'api/pymor/operators.html#to_matrix_xarraymatrixoperator',
                                                                                                 'pylgs/pymor/operators.py')},
            'pylgs.pymor.parameters': { 'pylgs.pymor.parameters.ExpressionParameterFunctional.__eq__': ( 'api/pymor/parameters.html#expressionparameterfunctional.__eq__',
                                                                                                         'pylgs/pymor/parameters.py'),
                                        'pylgs.pymor.parameters.ParameterFunctional.__str__': ( 'api/pymor/parameters.html#parameterfunctional.__str__',
                                                                                                'pylgs/pymor/parameters.py'),
                                        'pylgs.pymor.parameters.SimplifyFunctionalRules': ( 'api/pymor/parameters.html#simplifyfunctionalrules',
                                                                                            'pylgs/pymor/parameters.py'),
                                        'pylgs.pymor.parameters.SimplifyFunctionalRules.__init__': ( 'api/pymor/parameters.html#simplifyfunctionalrules.__init__',
                                                                                                     'pylgs/pymor/parameters.py'),
                                        'pylgs.pymor.parameters.SimplifyFunctionalRules.action_LincombOperator': ( 'api/pymor/parameters.html#simplifyfunctionalrules.action_lincomboperator',
                                                                                                                   'pylgs/pymor/parameters.py'),
                                        'pylgs.pymor.parameters.SimplifyFunctionalRules.action_ProductParameterFunctional': ( 'api/pymor/parameters.html#simplifyfunctionalrules.action_productparameterfunctional',
                                                                                                                              'pylgs/pymor/parameters.py'),
                                        'pylgs.pymor.parameters.SimplifyFunctionalRules.action_generic': ( 'api/pymor/parameters.html#simplifyfunctionalrules.action_generic',
                                                                                                           'pylgs/pymor/parameters.py'),
                                        'pylgs.pymor.parameters.SimplifyFunctionalRules.action_recurse': ( 'api/pymor/parameters.html#simplifyfunctionalrules.action_recurse',
                                                                                                           'pylgs/pymor/parameters.py'),
                                        'pylgs.pymor.parameters.simplify_functionals': ( 'api/pymor/parameters.html#simplify_functionals',
                                                                                         'pylgs/pymor/parameters.py')},
            'pylgs.pymor.timestepping': { 'pylgs.pymor.timestepping.AdamsTimeStepper': ( 'api/pymor/timestepping.html#adamstimestepper',
                                                                                         'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.AdamsTimeStepper.__init__': ( 'api/pymor/timestepping.html#adamstimestepper.__init__',
                                                                                                  'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.AdamsTimeStepper.iterate': ( 'api/pymor/timestepping.html#adamstimestepper.iterate',
                                                                                                 'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.BDFTimeStepper': ( 'api/pymor/timestepping.html#bdftimestepper',
                                                                                       'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.BDFTimeStepper.__init__': ( 'api/pymor/timestepping.html#bdftimestepper.__init__',
                                                                                                'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.BDFTimeStepper.iterate': ( 'api/pymor/timestepping.html#bdftimestepper.iterate',
                                                                                               'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.StiffSwitchingTimeStepper': ( 'api/pymor/timestepping.html#stiffswitchingtimestepper',
                                                                                                  'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.StiffSwitchingTimeStepper.__init__': ( 'api/pymor/timestepping.html#stiffswitchingtimestepper.__init__',
                                                                                                           'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.StiffSwitchingTimeStepper.iterate': ( 'api/pymor/timestepping.html#stiffswitchingtimestepper.iterate',
                                                                                                          'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.TimeStepper.solve': ( 'api/pymor/timestepping.html#timestepper.solve',
                                                                                          'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.init_cvode': ( 'api/pymor/timestepping.html#init_cvode',
                                                                                   'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.init_progress_widget': ( 'api/pymor/timestepping.html#init_progress_widget',
                                                                                             'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.solve_cvode': ( 'api/pymor/timestepping.html#solve_cvode',
                                                                                    'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.tick': ( 'api/pymor/timestepping.html#tick',
                                                                             'pylgs/pymor/timestepping.py'),
                                          'pylgs.pymor.timestepping.update_progress_widget': ( 'api/pymor/timestepping.html#update_progress_widget',
                                                                                               'pylgs/pymor/timestepping.py')},
            'pylgs.pymor.vectorarrays': { 'pylgs.pymor.vectorarrays.Lbl': ( 'api/pymor/vectorarrays.html#lbl',
                                                                            'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray': ( 'api/pymor/vectorarrays.html#xarrayvectorarray',
                                                                                          'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.__mul__': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.__mul__',
                                                                                                  'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray._repr_html_': ( 'api/pymor/vectorarrays.html#xarrayvectorarray._repr_html_',
                                                                                                      'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.array': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.array',
                                                                                                'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.array_labeled_as_source': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.array_labeled_as_source',
                                                                                                                  'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.coords': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.coords',
                                                                                                 'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.data': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.data',
                                                                                               'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.dims': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.dims',
                                                                                               'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.extended_coord_data': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.extended_coord_data',
                                                                                                              'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.extended_dims': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.extended_dims',
                                                                                                        'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.extended_shape': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.extended_shape',
                                                                                                         'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.item': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.item',
                                                                                               'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.reim': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.reim',
                                                                                               'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.shape': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.shape',
                                                                                                'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArray.visualize': ( 'api/pymor/vectorarrays.html#xarrayvectorarray.visualize',
                                                                                                    'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl',
                                                                                              'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.__init__': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.__init__',
                                                                                                       'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.__len__': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.__len__',
                                                                                                      'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl._coords': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl._coords',
                                                                                                      'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl._len': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl._len',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.amax': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.amax',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.append': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.append',
                                                                                                     'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.array': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.array',
                                                                                                    'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.axpy': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.axpy',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.axpy_copy': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.axpy_copy',
                                                                                                        'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.conj': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.conj',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.copy': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.copy',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.delete': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.delete',
                                                                                                     'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.dofs': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.dofs',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.imag': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.imag',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.inner': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.inner',
                                                                                                    'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.lincomb': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.lincomb',
                                                                                                      'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.norm': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.norm',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.norm2': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.norm2',
                                                                                                    'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.pairwise_inner': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.pairwise_inner',
                                                                                                             'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.real': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.real',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.reim': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.reim',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.scal': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.scal',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.scal_copy': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.scal_copy',
                                                                                                        'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.stacked_array': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.stacked_array',
                                                                                                            'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.stacked_data': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.stacked_data',
                                                                                                           'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorArrayImpl.to_numpy': ( 'api/pymor/vectorarrays.html#xarrayvectorarrayimpl.to_numpy',
                                                                                                       'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace': ( 'api/pymor/vectorarrays.html#xarrayvectorspace',
                                                                                          'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.__contains__': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.__contains__',
                                                                                                       'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.__eq__': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.__eq__',
                                                                                                 'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.__hash__': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.__hash__',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.__init__': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.__init__',
                                                                                                   'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.__mul__': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.__mul__',
                                                                                                  'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace._array': ( 'api/pymor/vectorarrays.html#xarrayvectorspace._array',
                                                                                                 'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace._array_factory': ( 'api/pymor/vectorarrays.html#xarrayvectorspace._array_factory',
                                                                                                         'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace._repr_html_': ( 'api/pymor/vectorarrays.html#xarrayvectorspace._repr_html_',
                                                                                                      'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.array_from_fn': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.array_from_fn',
                                                                                                        'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.concatenate': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.concatenate',
                                                                                                      'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.coords': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.coords',
                                                                                                 'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.dim': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.dim',
                                                                                              'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.from_data': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.from_data',
                                                                                                    'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.from_numpy': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.from_numpy',
                                                                                                     'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.from_xarray': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.from_xarray',
                                                                                                      'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.is_scalar': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.is_scalar',
                                                                                                    'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.make_array': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.make_array',
                                                                                                     'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.ones': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.ones',
                                                                                               'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.random': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.random',
                                                                                                 'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.rename': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.rename',
                                                                                                 'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.XarrayVectorSpace.zeros': ( 'api/pymor/vectorarrays.html#xarrayvectorspace.zeros',
                                                                                                'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.____': ( 'api/pymor/vectorarrays.html#____',
                                                                             'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays._categorical_dims': ( 'api/pymor/vectorarrays.html#_categorical_dims',
                                                                                          'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays._format_slider_labels': ( 'api/pymor/vectorarrays.html#_format_slider_labels',
                                                                                              'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays._numerical_dim_sort_order': ( 'api/pymor/vectorarrays.html#_numerical_dim_sort_order',
                                                                                                  'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays._numerical_dims': ( 'api/pymor/vectorarrays.html#_numerical_dims',
                                                                                        'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays._reim': ( 'api/pymor/vectorarrays.html#_reim',
                                                                              'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays._set_plotly_frame': ( 'api/pymor/vectorarrays.html#_set_plotly_frame',
                                                                                          'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.coord_data_shape': ( 'api/pymor/vectorarrays.html#coord_data_shape',
                                                                                         'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.dataarray_coordinate_data': ( 'api/pymor/vectorarrays.html#dataarray_coordinate_data',
                                                                                                  'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.default_coordinate': ( 'api/pymor/vectorarrays.html#default_coordinate',
                                                                                           'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.make_coordinates': ( 'api/pymor/vectorarrays.html#make_coordinates',
                                                                                         'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.plotly_dataarray': ( 'api/pymor/vectorarrays.html#plotly_dataarray',
                                                                                         'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.transpose_like': ( 'api/pymor/vectorarrays.html#transpose_like',
                                                                                       'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.unlabel_dims': ( 'api/pymor/vectorarrays.html#unlabel_dims',
                                                                                     'pylgs/pymor/vectorarrays.py'),
                                          'pylgs.pymor.vectorarrays.validate_coord_data': ( 'api/pymor/vectorarrays.html#validate_coord_data',
                                                                                            'pylgs/pymor/vectorarrays.py')},
            'pylgs.utilities.basic': { 'pylgs.utilities.basic.filter_args': ( 'api/utilities/basic.html#filter_args',
                                                                              'pylgs/utilities/basic.py'),
                                       'pylgs.utilities.basic.filter_out_args': ( 'api/utilities/basic.html#filter_out_args',
                                                                                  'pylgs/utilities/basic.py'),
                                       'pylgs.utilities.basic.get_item': ('api/utilities/basic.html#get_item', 'pylgs/utilities/basic.py')},
            'pylgs.utilities.formatting': { 'pylgs.utilities.formatting.prefix_format': ( 'api/utilities/formatting.html#prefix_format',
                                                                                          'pylgs/utilities/formatting.py'),
                                            'pylgs.utilities.formatting.significant': ( 'api/utilities/formatting.html#significant',
                                                                                        'pylgs/utilities/formatting.py'),
                                            'pylgs.utilities.formatting.significant_digits': ( 'api/utilities/formatting.html#significant_digits',
                                                                                               'pylgs/utilities/formatting.py')},
            'pylgs.utilities.nbdev': { 'pylgs.utilities.nbdev.AttributeTbl': ( 'api/utilities/nbdev.html#attributetbl',
                                                                               'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.AttributeTbl.__init__': ( 'api/utilities/nbdev.html#attributetbl.__init__',
                                                                                        'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.DictTbl': ('api/utilities/nbdev.html#dicttbl', 'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.DictTbl.__init__': ( 'api/utilities/nbdev.html#dicttbl.__init__',
                                                                                   'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.MarkdownTable': ( 'api/utilities/nbdev.html#markdowntable',
                                                                                'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.MarkdownTable.__eq__': ( 'api/utilities/nbdev.html#markdowntable.__eq__',
                                                                                       'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.MarkdownTable.__init__': ( 'api/utilities/nbdev.html#markdowntable.__init__',
                                                                                         'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.MarkdownTable._columns': ( 'api/utilities/nbdev.html#markdowntable._columns',
                                                                                         'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.MarkdownTable._hdr_list': ( 'api/utilities/nbdev.html#markdowntable._hdr_list',
                                                                                          'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.MarkdownTable._repr_markdown_': ( 'api/utilities/nbdev.html#markdowntable._repr_markdown_',
                                                                                                'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.MarkdownTable.hdr_str': ( 'api/utilities/nbdev.html#markdowntable.hdr_str',
                                                                                        'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev.MarkdownTable.params_str': ( 'api/utilities/nbdev.html#markdowntable.params_str',
                                                                                           'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev._bold': ('api/utilities/nbdev.html#_bold', 'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev._list2row': ( 'api/utilities/nbdev.html#_list2row',
                                                                            'pylgs/utilities/nbdev.py'),
                                       'pylgs.utilities.nbdev._split_at': ( 'api/utilities/nbdev.html#_split_at',
                                                                            'pylgs/utilities/nbdev.py')},
            'pylgs.utilities.numpy': { 'pylgs.utilities.numpy.sym_range': ( 'api/utilities/numpy.html#sym_range',
                                                                            'pylgs/utilities/numpy.py')},
            'pylgs.utilities.sparse': { 'pylgs.utilities.sparse.dense_toeplitz': ( 'api/utilities/sparse.html#dense_toeplitz',
                                                                                   'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.kron': ('api/utilities/sparse.html#kron', 'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.restrict_bandwidth': ( 'api/utilities/sparse.html#restrict_bandwidth',
                                                                                       'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.sparse': ('api/utilities/sparse.html#sparse', 'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.sparse2d': ( 'api/utilities/sparse.html#sparse2d',
                                                                             'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.sparse2d_diag': ( 'api/utilities/sparse.html#sparse2d_diag',
                                                                                  'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.sparse2d_identity': ( 'api/utilities/sparse.html#sparse2d_identity',
                                                                                      'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.sparse2d_kronecker_matrix': ( 'api/utilities/sparse.html#sparse2d_kronecker_matrix',
                                                                                              'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.sparse_diag': ( 'api/utilities/sparse.html#sparse_diag',
                                                                                'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.sparse_identity': ( 'api/utilities/sparse.html#sparse_identity',
                                                                                    'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.sparse_kronecker_matrix': ( 'api/utilities/sparse.html#sparse_kronecker_matrix',
                                                                                            'pylgs/utilities/sparse.py'),
                                        'pylgs.utilities.sparse.sparse_toeplitz': ( 'api/utilities/sparse.html#sparse_toeplitz',
                                                                                    'pylgs/utilities/sparse.py')},
            'pylgs.utilities.testing': { 'pylgs.utilities.testing.test_array': ( 'api/utilities/testing.html#test_array',
                                                                                 'pylgs/utilities/testing.py'),
                                         'pylgs.utilities.testing.test_da': ( 'api/utilities/testing.html#test_da',
                                                                              'pylgs/utilities/testing.py')},
            'pylgs.utilities.xarray': { 'pylgs.utilities.xarray.Coordinates.__add__': ( 'api/utilities/xarray.html#coordinates.__add__',
                                                                                        'pylgs/utilities/xarray.py'),
                                        'pylgs.utilities.xarray.Coordinates.complement': ( 'api/utilities/xarray.html#coordinates.complement',
                                                                                           'pylgs/utilities/xarray.py'),
                                        'pylgs.utilities.xarray.Coordinates.contain': ( 'api/utilities/xarray.html#coordinates.contain',
                                                                                        'pylgs/utilities/xarray.py'),
                                        'pylgs.utilities.xarray.Coordinates.intersection': ( 'api/utilities/xarray.html#coordinates.intersection',
                                                                                             'pylgs/utilities/xarray.py'),
                                        'pylgs.utilities.xarray.Coordinates.shape': ( 'api/utilities/xarray.html#coordinates.shape',
                                                                                      'pylgs/utilities/xarray.py'),
                                        'pylgs.utilities.xarray.Coordinates.size': ( 'api/utilities/xarray.html#coordinates.size',
                                                                                     'pylgs/utilities/xarray.py')},
            'pylgs.velocitygroups': { 'pylgs.velocitygroups.VelocityGroups': ( 'api/velocitygroups.html#velocitygroups',
                                                                               'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups.VelocityGroups.__init__': ( 'api/velocitygroups.html#velocitygroups.__init__',
                                                                                        'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups.VelocityGroups.drho_dv': ( 'api/velocitygroups.html#velocitygroups.drho_dv',
                                                                                       'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups.VelocityGroups.identity': ( 'api/velocitygroups.html#velocitygroups.identity',
                                                                                        'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups.VelocityGroups.n_times_1': ( 'api/velocitygroups.html#velocitygroups.n_times_1',
                                                                                         'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups.VelocityGroups.normalize': ( 'api/velocitygroups.html#velocitygroups.normalize',
                                                                                         'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups.VelocityGroups.subdivide': ( 'api/velocitygroups.html#velocitygroups.subdivide',
                                                                                         'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups.VelocityGroups.sum': ( 'api/velocitygroups.html#velocitygroups.sum',
                                                                                   'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups.VelocityGroups.velocity_diagonal': ( 'api/velocitygroups.html#velocitygroups.velocity_diagonal',
                                                                                                 'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups._velocity_space': ( 'api/velocitygroups.html#_velocity_space',
                                                                                'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups._vg_da': ('api/velocitygroups.html#_vg_da', 'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups._vg_diagonal': ( 'api/velocitygroups.html#_vg_diagonal',
                                                                             'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups._vg_identity': ( 'api/velocitygroups.html#_vg_identity',
                                                                             'pylgs/velocitygroups.py'),
                                      'pylgs.velocitygroups.velocity_density_vector': ( 'api/velocitygroups.html#velocity_density_vector',
                                                                                        'pylgs/velocitygroups.py')}}}



"""Extended functionality for [pyMOR](https://pymor.org/) grids"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/api/pymor/grids.ipynb.

# %% auto 0
__all__ = ['OnedUnstructuredGrid']

# %% ../../nbs/api/pymor/grids.ipynb
from pymor.discretizers.builtin.grids.oned import OnedGrid
import numpy as np

# %% ../../nbs/api/pymor/grids.ipynb
class OnedUnstructuredGrid(OnedGrid):
    """One-dimensional |Grid| on an interval.

    Parameters
    ----------
    edges
        Sequence specifying the edges of the grid intervals.
    identify_left_right
        Whether the left and right edges of the grid should be identified.
    """

    def __init__(self, edges, identify_left_right=False):
        self.edges = np.array(edges)
        self.domain = (self.edges[0], self.edges[-1])
        assert self.edges.ndim == 1
        assert np.all(self.edges[:-1] <= self.edges[1:])
        self.__auto_init(locals())
        num_intervals = len(self.edges) - 1
        self._sizes = [num_intervals, num_intervals] if identify_left_right else [num_intervals, num_intervals + 1]
        self._widths = np.diff(self.edges)
        self._subentities = np.vstack((np.arange(num_intervals, dtype=np.int32),
                                        np.arange(num_intervals, dtype=np.int32) + 1))
        if identify_left_right:
            self._subentities[-1, -1] = 0
        self._A = self._widths[:, np.newaxis, np.newaxis]
        self._B = np.cumsum(np.pad(self._widths[:-1], (1, 0)))[:, np.newaxis] + self.domain[0]

    # The following methods would not need to be redefined from the parent class, 
    # except that name-mangling (double underscore) is used in the parent.
    # Ask about that and/or submit PR?
    
    def subentities(self, codim, subentity_codim):
        assert 0 <= codim <= 1, 'Invalid codimension'
        assert codim <= subentity_codim <= self.dim, 'Invalid subentity codimension'
        if codim == 0:
            if subentity_codim == 0:
                return np.arange(self.size(0), dtype='int32')[:, np.newaxis]
            else:
                return self._subentities.T
        else:
            return super().subentities(codim, subentity_codim)

    def embeddings(self, codim):
        if codim == 0:
            return self._A, self._B
        else:
            return super().embeddings(codim)



"""Extra functionality for [sparse](https://sparse.pydata.org) and [scipy.sparse](https://docs.scipy.org/doc/scipy/reference/sparse.html)"""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/api/utilities/sparse.ipynb.

# %% auto 0
__all__ = ['sparse2d_rand', 'spilu', 'sparse2d', 'sparse', 'sparse2d_identity', 'sparse_identity', 'sparse2d_diag', 'sparse_diag',
           'dense_toeplitz', 'sparse_toeplitz', 'sparse2d_kronecker_matrix', 'sparse_kronecker_matrix', 'kron',
           'restrict_bandwidth']

# %% ../../nbs/api/utilities/sparse.ipynb
import itertools as it

from fastcore.meta import delegates

import numpy as np
from numpy import array, ndarray
import scipy.sparse as sps
from scipy.sparse import sparray
import scipy.linalg as spl
import scipy.sparse.linalg as spa
from sparse import COO, GCXS, SparseArray
import sparse as ss

# %% ../../nbs/api/utilities/sparse.ipynb
def sparse2d(a):
    """Create a 2D sparse array in specified format."""
    if isinstance(a, SparseArray): a = a.to_scipy_sparse()
    return sps.csr_array(a)

# %% ../../nbs/api/utilities/sparse.ipynb
@delegates(COO, but='data')
def sparse(a, format='coo', shape=None, **kwargs)->SparseArray:
    """Create an N-D sparse array in specified format."""
    if format == 'gcxs': cls = GCXS
    elif format == 'coo': cls = COO
    if isinstance(a, ndarray): return cls.from_numpy(a)
    if sps.issparse(a): return cls.from_scipy_sparse(a)
    if isinstance(a, SparseArray): return cls(a)
    # Assume a is coords and data in COO format
    if shape is None: shape = [max(c) - min(c) + 1 for c in a[0]]
    return cls(COO(*a, shape=shape, **kwargs))

# %% ../../nbs/api/utilities/sparse.ipynb
def sparse2d_identity(n):
    return sps.eye_array(n)

# %% ../../nbs/api/utilities/sparse.ipynb
def sparse_identity(n, format='coo')->SparseArray:
    return ss.eye(n, format=format)

# %% ../../nbs/api/utilities/sparse.ipynb
def sparse2d_diag(diags, offsets=0):
    return sps.diags_array(diags, offsets=offsets)

# %% ../../nbs/api/utilities/sparse.ipynb
def sparse_diag(diag, format='coo')->SparseArray:
    return sparse(sparse2d_diag(diag), format=format)

# %% ../../nbs/api/utilities/sparse.ipynb
def dense_toeplitz(
    diags:ndarray, # Value on each diagonal, starting at lower left.
):
    """Return a toeplitz array given the value on each diagonal (starting at lower left)."""
    n = (len(diags) - 1) // 2
    return spl.toeplitz(diags[:n+1][::-1], diags[n:])

# %% ../../nbs/api/utilities/sparse.ipynb
def sparse_toeplitz(
    diags:ndarray, # Value on each diagonal, starting at lower left.
    format='coo'
)->SparseArray:
    """Return a toeplitz sparse array in specified format given the value on each diagonal (starting at lower left)."""
    return sparse(dense_toeplitz(diags), format=format)

# %% ../../nbs/api/utilities/sparse.ipynb
def sparse2d_kronecker_matrix(n, k=0):
    """The nxn sparse kronecker matrix delta_(i,j-k) in specified format."""
    return sparse2d_diag(np.ones(n - abs(k)), offsets=k)

# %% ../../nbs/api/utilities/sparse.ipynb
def sparse_kronecker_matrix(n, k=0, format='coo')->SparseArray:
    """The nxn sparse kronecker matrix delta_(i,j-k) in specified format."""
    return sparse(sparse2d_kronecker_matrix(n, k), format=format)

# %% ../../nbs/api/utilities/sparse.ipynb
def kron(a, b, *rest):
    if rest: return kron(kron(a, b), *rest)
    if sps.issparse(a) or sps.issparse(b): 
        return sps.kron(a, b, format='csr')
    return np.kron(a, b)

# %% ../../nbs/api/utilities/sparse.ipynb
sparse2d_rand = sps.random_array

# %% ../../nbs/api/utilities/sparse.ipynb
spilu = spa.spilu

# %% ../../nbs/api/utilities/sparse.ipynb
def restrict_bandwidth(a_sparray:sparray, width):
    """Make a scipy sparse array banded by setting all elements outside the bandwidth to zero."""
    result = a_sparray.copy()
    i, j = result.nonzero()
    result.data[np.abs(i - j) > width] = 0.
    result.eliminate_zeros()
    return result
